# -*- coding: utf-8 -*-
"""
Clients used to access third party systems.
"""

from __future__ import absolute_import, unicode_literals

import os
from time import time
from functools import wraps
from six.moves import urllib

from edx_rest_api_client.client import EdxRestApiClient


class EdxOAuth2APIClient(object):
    """
    Base API Client for accessing edX IDA API endpoints.
    """

    LMS_ROOT_URL = os.environ.get('LMS_ROOT_URL')
    LMS_OAUTH_HOST = os.environ.get('LMS_OAUTH_HOST')
    API_BASE_URL = LMS_ROOT_URL + '/api/'
    APPEND_SLASH = False

    def __init__(self, client_id, client_secret):
        """
        Connect to the REST API.
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.expires_at = 0
        self.client = None

    def connect(self):
        """
        Connect to the REST API, authenticating with an access token retrieved with our client credentials.
        """

        access_token, expires_at = EdxRestApiClient.get_oauth_access_token(
            self.LMS_OAUTH_HOST + '/oauth2/access_token',
            self.client_id,
            self.client_secret,
            'jwt'
        )
        self.client = EdxRestApiClient(
            self.API_BASE_URL, append_slash=self.APPEND_SLASH, jwt=access_token,
        )
        self.expires_at = expires_at

    def token_expired(self):
        """
        Return True if the JWT token has expired, False if not.
        """
        return int(time()) > self.expires_at

    @staticmethod
    def refresh_token(func):
        """
        Use this method decorator to ensure the JWT token is refreshed when needed.
        """
        @wraps(func)
        def inner(self, *args, **kwargs):
            """
            Before calling the wrapped function, we check if the JWT token is expired, and if so, re-connect.
            """
            if self.token_expired():
                self.connect()
            return func(self, *args, **kwargs)
        return inner


def traverse_pagination(response, endpoint):
    """
    Traverse a paginated API response.

    Extracts and concatenates "results" (list of dict) returned by DRF-powered
    APIs.

    Arguments:
        response (Dict): Current response dict from service API
        endpoint (slumber Resource object): slumber Resource object from edx-rest-api-client

    Returns:
        list of dict.

    """
    results = response.get('results', [])

    next_page = response.get('next')
    while next_page:
        querystring = urllib.parse.parse_qs(urllib.parse.urlparse(next_page).query, True)
        response = endpoint.get(**querystring)
        results += response.get('results', [])
        next_page = response.get('next')

    return results



# -*- coding: utf-8 -*-
"""
Utility functions for Enterprise Reporting.
"""
from __future__ import absolute_import, unicode_literals

import datetime
import logging
import os
import re
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from io import open  # pylint: disable=redefined-builtin

import boto3
import pyminizip
import pytz
from cryptography.fernet import Fernet
from fernet_fields.hkdf import derive_fernet_key

from django.utils.encoding import force_text

LOGGER = logging.getLogger(__name__)

COMPRESSION_LEVEL = 4

FREQUENCY_TYPE_DAILY = 'daily'
FREQUENCY_TYPE_MONTHLY = 'monthly'
FREQUENCY_TYPE_WEEKLY = 'weekly'

AWS_REGION = 'us-east-1'


def compress_and_encrypt(filename, password):
    """
    Given a file and a password, create an encrypted zip file. Return the new filename.
    """
    zip_filename = re.sub(r'\.(\w+)$', '.zip', filename)
    if filename == zip_filename:
        LOGGER.warn('Unable to determine filename for compressing {}, '
                    'file must have a valid extension that is not .zip'.format(filename))
        return None

    pyminizip.compress(filename, zip_filename, password, COMPRESSION_LEVEL)
    return zip_filename


def send_email_with_attachment(subject, body, from_email, to_email, filename):
    """
    Send an email with a file attachment.

    Adapted from https://gist.github.com/yosemitebandit/2883593
    """
    # connect to SES
    client = boto3.client('ses', region_name=AWS_REGION)

    # the message body
    msg_body = MIMEText(body)
    
    # the attachment
    msg_attachment = MIMEApplication(open(filename, 'rb').read())
    msg_attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(filename))
    msg_attachment.set_type('application/zip')

    # iterate over each email in the list to send emails independently
    for email in to_email:
        msg = MIMEMultipart()
        msg['Subject'] = subject
        msg['From'] = from_email
        msg['To'] = email

        # what a recipient sees if they don't use an email reader
        msg.preamble = 'Multipart message.\n'

        # attach the message body and attachment
        msg.attach(msg_body)
        msg.attach(msg_attachment)

        # and send the message
        result = client.send_raw_email(RawMessage={'Data': msg.as_string()}, Source=msg['From'], Destinations=[email])
        LOGGER.debug(result)


def is_current_time_in_schedule(frequency, hour_of_day, day_of_month=None, day_of_week=None):
    """
    Determine if the current time is in the range specified by this configuration's schedule.
    """
    est_timezone = pytz.timezone('US/Eastern')
    current_est_time = datetime.datetime.now(est_timezone)
    current_hour_of_day = current_est_time.hour
    current_day_of_week = current_est_time.weekday()
    current_day_of_month = current_est_time.day

    # All configurations have an hour of the day, so the hour must always match in order to send a report.
    if hour_of_day == current_hour_of_day:
        # If reports should be sent monthly and today is the same as the day configured, return True
        if frequency == FREQUENCY_TYPE_MONTHLY and day_of_month == current_day_of_month:
            return True
        # If reports should be sent weekly and today is the same as the day configured, return True
        elif frequency == FREQUENCY_TYPE_WEEKLY and day_of_week == current_day_of_week:
            return True
        # If reports should be sent daily, return True
        elif frequency == FREQUENCY_TYPE_DAILY:
            return True

    return False


def decrypt_string(string):
    """
    Decrypts a string that was encrypted using Fernet symmetric encryption.
    """
    fernet = Fernet(
        derive_fernet_key(
            os.environ.get('LMS_FERNET_KEY')
        )
    )
    return force_text(fernet.decrypt(bytes(string, 'utf-8')))



"""
Permissions needed to restrict access to the enterprise data api.
"""
from __future__ import absolute_import, unicode_literals

from logging import getLogger

from rest_framework import permissions

from enterprise_data.clients import EnterpriseApiClient

LOGGER = getLogger(__name__)


class IsStaffOrEnterpriseUser(permissions.BasePermission):
    """
    Permission that checks to see if the request user is staff or is associated with the enterprise in the request.

    NOTE: This permission check may make a request to the LMS to get the enterprise association if it is not already in
    the session. This fetch should go away when JWT Scopes are fully implemented and the association is stored on
    the token.
    """

    def get_user_enterprise_id(self, auth_token, user):
        """
        Get the enterprise learner model from the LMS for the given user.

        Returns: learner or None if unable to get or user is not associated with an enterprise
        """
        enterprise_client = EnterpriseApiClient(auth_token)
        enterprise_learner_data = enterprise_client.get_enterprise_learner(user)
        if not enterprise_learner_data:
            return None

        return enterprise_learner_data['enterprise_customer']['uuid']

    def has_permission(self, request, view):
        """
        Verify the user is staff or the associated enterprise matches the requested enterprise.
        """
        if request.user.is_staff:
            return True

        if not hasattr(request.session, 'enterprise_id'):
            request.session['enterprise_id'] = self.get_user_enterprise_id(request.auth, request.user)

        enterprise_in_url = request.parser_context.get('kwargs', {}).get('enterprise_id', '')

        permitted = request.session['enterprise_id'] == enterprise_in_url
        if not permitted:
            LOGGER.warning('User {} denied access to EnterpriseEnrollments for enterprise {}'
                           .format(request.user, enterprise_in_url))

        return permitted



# -*- coding: utf-8 -*-
"""
Tests for views in the `enterprise_data` module.
"""
from __future__ import absolute_import, unicode_literals

from pytest import mark
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase

from test_utils import UserFactory


@mark.django_db
class TestEnterpriseEnrollmentsView(APITestCase):
    """
    Tests for EnterpriseEnrollmentsView
    """
    fixtures = ('enterprise_enrollment',)

    def setUp(self):
        super(TestEnterpriseEnrollmentsView, self).setUp()
        self.user = UserFactory(is_staff=True)
        self.client.force_authenticate(user=self.user)
        self.url = reverse('v0:enterprise_enrollments',
                           kwargs={'enterprise_id': 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'})

    def test_get_queryset_returns_enrollments(self):
        expected_result = {
            'count': 1,
            'num_pages': 1,
            'current_page': 1,
            'results': [{
                'enrollment_created_timestamp': '2014-06-27T16:02:38',
                'user_current_enrollment_mode': 'verified',
                'has_passed': True,
                'course_id': 'edX/Open_DemoX/edx_demo_course',
                'id': 2,
                'course_min_effort': 2,
                'course_start': '2016-09-01T00:00:00',
                'enterprise_user_id': 1,
                'course_title': 'All about acceptance testing!',
                'course_duration_weeks': '8',
                'course_pacing_type': 'instructor_paced',
                'user_username': 'test_user',
                'enterprise_sso_uid': 'harry',
                'enterprise_site_id': None,
                'enterprise_id': 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c',
                'course_end': '2016-12-01T00:00:00',
                'lms_user_id': 11,
                'enterprise_name': 'Enterprise 1',
                'letter_grade': 'Pass',
                'user_account_creation_timestamp': '2015-02-12T23:14:35',
                'passed_timestamp': '2017-05-09T16:27:34.690065',
                'course_max_effort': 4,
                'consent_granted': True,
                'user_email': 'test@example.com'
            }],
            'next': None,
            'start': 0,
            'previous': None
        }

        response = self.client.get(self.url)
        assert response.status_code == status.HTTP_200_OK
        result = response.json()
        assert result == expected_result



# -*- coding: utf-8 -*-
"""
Serializers for enterprise api version 0 endpoint.
"""
from __future__ import absolute_import, unicode_literals

from rest_framework import serializers

from enterprise_data.models import EnterpriseEnrollment


class EnterpriseEnrollmentSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseEnrollment model.
    """

    class Meta:
        model = EnterpriseEnrollment
        fields = '__all__'



# -*- coding: utf-8 -*-
"""
URL definitions for enterprise data api version 1 endpoint.
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from enterprise_data.api.v0 import views

urlpatterns = [
    url(r'^enterprise/(?P<enterprise_id>.+)/enrollments/$',
        views.EnterpriseEnrollmentsView.as_view(),
        name='enterprise_enrollments'),
]



# -*- coding: utf-8 -*-
"""
Classes that handle sending reports for EnterpriseCustomers.
"""
from __future__ import absolute_import, unicode_literals

import csv
import datetime
import logging
import os
from io import open  # pylint: disable=redefined-builtin
from smtplib import SMTPException
from uuid import UUID

import paramiko
from enterprise_reporting.clients.vertica import VerticaClient
from enterprise_reporting.utils import compress_and_encrypt, decrypt_string, send_email_with_attachment

LOGGER = logging.getLogger(__name__)


class EnterpriseReportSenderFactory(object):
    """
    Factory that creates the EnterpriseReportSender given a reporting configuration.
    """

    @staticmethod
    def create(reporting_config):
        """
        Create the EnterpriseReportSender and all of its dependencies.
        """
        enterprise_customer_name = reporting_config['enterprise_customer']['name']
        vertica_client = VerticaClient(
            os.environ.get('VERTICA_HOST'),
            os.environ.get('VERTICA_USERNAME'),
            os.environ.get('VERTICA_PASSWORD')
        )
        if reporting_config['delivery_method'] == 'email':
            LOGGER.debug('{} is configured to send the report via email to {}'.format(
                enterprise_customer_name,
                reporting_config['email'],
            ))
            delivery_method = EmailDeliveryMethod(
                reporting_config['email'],
                decrypt_string(reporting_config['encrypted_password']),
                enterprise_customer_name
            )
        elif reporting_config['delivery_method'] == 'sftp':
            LOGGER.debug('{} is configured to send the report via sftp'.format(enterprise_customer_name))
            delivery_method = SFTPDeliveryMethod(
                reporting_config['sftp_hostname'],
                reporting_config['sftp_port'],
                reporting_config['sftp_username'],
                decrypt_string(reporting_config['encrypted_sftp_password']),
                reporting_config['sftp_file_path'],
                enterprise_customer_name
            )
        else:
            raise Exception

        return EnterpriseReportSender(reporting_config, vertica_client, delivery_method)


class EnterpriseReportSender(object):
    """
    Class that handles the process of sending a data report to an Enterprise Customer.
    """

    VERTICA_QUERY = ("SELECT {fields} FROM business_intelligence.enterprise_enrollment"
                     " WHERE enterprise_id = '{enterprise_id}' AND consent_granted = 1")
    VERTICA_QUERY_FIELDS = (
        'enterprise_user_id',
        'lms_user_id',
        'enterprise_sso_uid',
        'enrollment_created_timestamp',
        'consent_granted',
        'course_id',
        'course_title',
        'course_duration',
        'course_min_effort',
        'course_max_effort',
        'user_account_creation_date',
        'user_email',
        'user_username',
        'user_age',
        'user_level_of_education',
        'user_gender',
        'user_country_code',
        'country_name',
        'has_passed',
        'passed_timestamp',
        'time_spent_hours',
        'last_activity_date',
        'user_current_enrollment_mode',
    )
    REPORT_FILE_NAME_FORMAT = "{path}/{enterprise_id}_{date}.{extension}"
    FILE_WRITE_DIRECTORY = '/tmp'

    def __init__(self, reporting_config, vertica_client, delivery_method):
        """
        Initialize with an EnterpriseCustomerReportingConfiguration.
        """
        self.reporting_config = reporting_config
        self.vertica_client = vertica_client
        self.delivery_method = delivery_method

    def send_enterprise_report(self):
        """
        Query the data warehouse (vertica) and export data to a csv file.

        This file will get encrypted and emailed to the Enterprise Customer.
        """
        enterprise_customer_name = self.reporting_config['enterprise_customer']['name']

        LOGGER.info('Starting process to send report to {}'.format(enterprise_customer_name))

        # Query vertica and write output to csv file.
        self.vertica_client.connect()
        data_report_file_name = self.REPORT_FILE_NAME_FORMAT.format(
            path=self.FILE_WRITE_DIRECTORY,
            enterprise_id=self.reporting_config['enterprise_customer']['uuid'],
            date=datetime.datetime.now().strftime("%Y-%m-%d"),
            extension='csv',
        )
        with open(data_report_file_name, 'w') as data_report_file:
            data_report_file_writer = csv.writer(data_report_file)
            data_report_file_writer.writerow(self.VERTICA_QUERY_FIELDS)
            LOGGER.debug('Querying Vertica for data for {}'.format(enterprise_customer_name))
            data_report_file_writer.writerows(self._query_vertica())

        # Send the data based on the delivery method.
        self.delivery_method.send(data_report_file_name)

        self._cleanup()

    def _query_vertica(self):
        """
        Use a connection to Vertica to execute the report query.
        """
        query = self.VERTICA_QUERY.format(
            fields=','.join(self.VERTICA_QUERY_FIELDS),
            enterprise_id=UUID(self.reporting_config['enterprise_customer']['uuid']).hex
        )
        LOGGER.debug('Executing Vertica query: {}'.format(query))
        return self.vertica_client.stream_results(query)

    def _cleanup(self):
        """
        Perform various cleanup operations after we've attempted (successfully or not) to send a report.
        """
        self.vertica_client.close_connection()


class EmailDeliveryMethod(object):
    """
    Class that handles sending an enterprise report file via email as an encrypted zip file.
    """

    REPORT_EMAIL_SUBJECT = '{enterprise_name} edX Learner Data'
    REPORT_EMAIL_BODY = """
    Please find attached employee progress data for courses on edX.
    For any questions or concerns, please contact your edX sales representative.
    Thanks,
    The edX for Business Team
    """
    REPORT_EMAIL_FROM_EMAIL = os.environ.get('SEND_EMAIL_FROM')

    def __init__(self, email, password, enteprise_customer_name):
        """
        Initialize the Email Delivery Method.
        """
        self.email = email if isinstance(email, list) else [email] # convert to list if it's not already
        self.password = password
        self.enterprise_customer_name = enteprise_customer_name

    def send(self, enterprise_report_file_name):
        """
        Send the given file with the configured information.
        """
        # create a password encrypted zip file
        LOGGER.info('Encrypting data report for {}'.format(self.enterprise_customer_name))
        data_report_zip_name = compress_and_encrypt(enterprise_report_file_name, self.password)

        data_report_subject = self.REPORT_EMAIL_SUBJECT.format(
            enterprise_name=self.enterprise_customer_name
        )

        # email the file to the email address(es) in the configuration
        LOGGER.info('Sending encrypted data to {}'.format(self.enterprise_customer_name))
        try:
            send_email_with_attachment(
                data_report_subject,
                self.REPORT_EMAIL_BODY,
                self.REPORT_EMAIL_FROM_EMAIL,
                self.email,
                data_report_zip_name
            )
            LOGGER.info('Email report with encrypted zip successfully sent to {} for {}'.format(
                ', '.join(self.email),
                self.enterprise_customer_name
            ))
        except SMTPException:
            LOGGER.exception('Failed to send email report to {} for {}'.format(
                self.email,
                self.enterprise_customer_name
            ))


class SFTPDeliveryMethod(object):
    """
    Class that handles sending an enterprise report file via SFTP.
    """

    def __init__(self, hostname, port, username, password, file_path, enterprise_customer_name):
        """
        Initialize the SFTP Delivery Method.
        """
        self.hostname = hostname
        self.port = port
        self.username = username
        self.password = password
        self.file_path = file_path
        self.enterprise_customer_name = enterprise_customer_name

    def send(self, enterprise_report_file_name):
        """
        Send the given file with the configured information.
        """
        LOGGER.info('Connecting via sftp to remote host {} for {}'.format(
            self.hostname,
            self.enterprise_customer_name
        ))
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(
            hostname=self.hostname,
            port=self.port,
            username=self.username,
            password=self.password,
        )
        sftp = ssh.open_sftp()
        sftp.put(
            enterprise_report_file_name,
            os.path.join(self.file_path, os.path.basename(enterprise_report_file_name))
        )
        sftp.close()
        ssh.close()
        LOGGER.info('Successfully sent report via sftp for {}'.format(self.enterprise_customer_name))



# -*- coding: utf-8 -*-
"""
Views for enterprise api version 0 endpoint.
"""
from __future__ import absolute_import, unicode_literals

from edx_rest_framework_extensions.authentication import JwtAuthentication
from edx_rest_framework_extensions.paginators import DefaultPagination
from rest_framework import generics

from enterprise_data.api.v0 import serializers
from enterprise_data.filters import ConsentGrantedFilterBackend
from enterprise_data.models import EnterpriseEnrollment
from enterprise_data.permissions import IsStaffOrEnterpriseUser


class EnterpriseEnrollmentsView(generics.ListAPIView):
    """
    The EnterpriseEnrollments view returns all learner enrollment records for a given enterprise
    """
    serializer_class = serializers.EnterpriseEnrollmentSerializer
    pagination_class = DefaultPagination
    authentication_classes = (JwtAuthentication,)
    permission_classes = (IsStaffOrEnterpriseUser,)
    filter_backends = (ConsentGrantedFilterBackend,)
    CONSENT_GRANTED_FILTER = 'consent_granted'

    def get_queryset(self):
        enterprise_id = self.kwargs['enterprise_id']
        return EnterpriseEnrollment.objects.filter(enterprise_id=enterprise_id)



# -*- coding: utf-8 -*-
"""
Client for connecting to the LMS Enterprise endpoints.
"""

from __future__ import absolute_import, unicode_literals

from enterprise_reporting.clients import EdxOAuth2APIClient, traverse_pagination


class EnterpriseAPIClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Enterprise API.
    """

    API_BASE_URL = EdxOAuth2APIClient.LMS_ROOT_URL + '/enterprise/api/v1/'
    APPEND_SLASH = True

    ENTERPRISE_REPORTING_ENDPOINT = 'enterprise_customer_reporting'

    DEFAULT_VALUE_SAFEGUARD = object()

    def get_all_enterprise_reporting_configs(self, **kwargs):
        """
        Query the enterprise customer reporting endpoint for all available configs.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            should_traverse_pagination=True,
            **kwargs
        )

    def get_enterprise_reporting_config(self, enterprise_customer_id, **kwargs):
        """
        Query the enterprise customer reporting endpoint for a given enterprise customer.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            querystring={'enterprise_customer': enterprise_customer_id},
            **kwargs
        )

    @EdxOAuth2APIClient.refresh_token
    def _load_data(
            self,
            resource,
            detail_resource=None,
            resource_id=None,
            querystring=None,
            should_traverse_pagination=False,
            default=DEFAULT_VALUE_SAFEGUARD,
    ):
        """
        Loads a response from a call to one of the Enterprise endpoints.

        Arguments:
            resource: The endpoint resource name.
            detail_resource: The sub-resource to append to the path.
            resource_id: The resource ID for the specific detail to get from the endpoint.
            querystring: Optional query string parameters.
            should_traverse_pagination: Whether to traverse pagination or return paginated response.
            default: The default value to return in case of no response content.

        Returns
            (JSON): Data returned by the API.
        """
        default_val = default if default is not self.DEFAULT_VALUE_SAFEGUARD else {}
        querystring = querystring or {}

        endpoint = getattr(self.client, resource)
        endpoint = getattr(self.client, resource)(resource_id) if resource_id else endpoint
        endpoint = getattr(endpoint, detail_resource) if detail_resource else endpoint
        response = endpoint.get(**querystring)
        if should_traverse_pagination:
            results = traverse_pagination(response, endpoint)
            response = {
                'count': len(results),
                'next': None,
                'previous': None,
                'results': results,
            }

        return response or default_val



#!/usr/bin/python3
"""
Sends an Enterprise Customer's data file to a configured destination.
"""

from __future__ import absolute_import, unicode_literals

import argparse
import logging
import os
import re
import sys

from enterprise_reporting.clients.enterprise import EnterpriseAPIClient
from enterprise_reporting.reporter import EnterpriseReportSender, EnterpriseReportSenderFactory
from enterprise_reporting.utils import is_current_time_in_schedule

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


def send_data(config):
    """
    Send data report to each enterprise.

    Args:
        config
    """
    enterprise_customer_name = config['enterprise_customer']['name']
    LOGGER.info('Kicking off job to send report for {}'.format(
        enterprise_customer_name
    ))

    try:
        reporter = EnterpriseReportSenderFactory.create(config)
        reporter.send_enterprise_report()
    except Exception:  # pylint: disable=broad-except
        exception_message = 'Data report failed to send for {enterprise_customer}'.format(
            enterprise_customer=enterprise_customer_name,
        )
        LOGGER.exception(exception_message)

    cleanup_files(config['enterprise_customer']['uuid'])
    LOGGER.info('Finished job to send report for {}'.format(
        enterprise_customer_name
    ))


def cleanup_files(enterprise_id):
    """
    Clean up any files created by sending the enterprise report.
    """
    directory = EnterpriseReportSender.FILE_WRITE_DIRECTORY
    pattern = r'{}'.format(enterprise_id)
    for f in os.listdir(directory):
        if re.search(pattern, f):
            os.remove(os.path.join(directory, f))


def process_reports():
    """
    Process and send reports based on the arguments passed.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-e', '--enterprise-customer', required=False, type=str,
                        help="EnterpriseCustomer UUID.")

    args = parser.parse_args()
    enterprise_api_client = EnterpriseAPIClient(
        os.environ.get('LMS_OAUTH_KEY'),
        os.environ.get('LMS_OAUTH_SECRET')
    )

    if args.enterprise_customer:
        reporting_configs = enterprise_api_client.get_enterprise_reporting_config(args.enterprise_customer)
    else:
        reporting_configs = enterprise_api_client.get_all_enterprise_reporting_configs()

    if args.enterprise_customer and not (reporting_configs and reporting_configs['results']):
        LOGGER.error('The enterprise {} does not have a reporting configuration.'.format(args.enterprise_customer))
        sys.exit(1)

    for reporting_config in reporting_configs['results']:
        LOGGER.info('Checking if reporting config for {} is ready for processing'.format(
            reporting_config['enterprise_customer']['name']
        ))
        if (args.enterprise_customer or
                is_current_time_in_schedule(
                    reporting_config['frequency'],
                    reporting_config['hour_of_day'],
                    reporting_config['day_of_month'],
                    reporting_config['day_of_week'])):
            send_data(reporting_config)


if __name__ == "__main__":
    process_reports()
    sys.exit(0)



# -*- coding: utf-8 -*-
"""
Tests for the `edx-enterprise` serializer module.
"""

from __future__ import absolute_import, unicode_literals

from pytest import mark, raises
from rest_framework.test import APITestCase

from enterprise_data.api.v0.serializers import EnterpriseEnrollmentSerializer


@mark.django_db
class TestEnterpriseEnrollmentSerializer(APITestCase):
    """
    Tests for `enterprise_enrollment` API serializer.
    """

    def setUp(self):
        super(TestEnterpriseEnrollmentSerializer, self).setUp()

        self.enrollment_data = {
            "id": 2,
            "enterprise_id": "ee5e6b3a-069a-4947-bb8d-d2dbc323396c",
            "enterprise_name": "Enterprise 1",
            "lms_user_id": 11,
            "enterprise_user_id": 1,
            "course_id": "edX/Open_DemoX/edx_demo_course",
            "enrollment_created_timestamp": "2014-06-27T16:02:38Z",
            "user_current_enrollment_mode": "verified",
            "consent_granted": 1,
            "letter_grade": "Pass",
            "has_passed": 1,
            "passed_timestamp": "2017-05-09T16:27:34.690065Z",
            "enterprise_sso_uid": "harry",
            "course_title": "All about acceptance testing!",
            "course_start": "2016-09-01T00:00:00Z",
            "course_end": "2016-12-01T00:00:00Z",
            "course_pacing_type": "instructor_paced",
            "course_duration_weeks": "8",
            "course_min_effort": 2,
            "course_max_effort": 4,
            "user_account_creation_timestamp": "2015-02-12T23:14:35Z",
            "user_email": "test@example.com",
            "user_username": "test_user"
        }

    def test_enrollment_serialization(self):
        expected_serialized_data = {
            'id': 2,
            'enterprise_id': 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c',
            'enterprise_name': 'Enterprise 1',
            'lms_user_id': 11,
            'enterprise_user_id': 1,
            'course_id': 'edX/Open_DemoX/edx_demo_course',
            'enrollment_created_timestamp': '2014-06-27T16:02:38Z',
            'user_current_enrollment_mode': 'verified',
            'consent_granted': True,
            'letter_grade': 'Pass',
            'has_passed': True,
            'course_min_effort': 2,
            'course_start': '2016-09-01T00:00:00Z',
            'course_title': 'All about acceptance testing!',
            'course_duration_weeks': '8',
            'course_pacing_type': 'instructor_paced',
            'enterprise_sso_uid': 'harry',
            'course_end': '2016-12-01T00:00:00Z',
            'user_account_creation_timestamp': '2015-02-12T23:14:35Z',
            'passed_timestamp': '2017-05-09T16:27:34.690065Z',
            'course_max_effort': 4,
            'user_email': 'test@example.com',
            'user_username': 'test_user'
        }
        serializer = EnterpriseEnrollmentSerializer(self.enrollment_data)
        assert serializer.data == expected_serialized_data



# -*- coding: utf-8 -*-
"""
Tests for the `enterprise-data` models module.
"""
import unittest

import ddt
from pytest import mark

from test_utils import EnterpriseEnrollmentFactory


@mark.django_db
@ddt.ddt
class TestEnterpriseEnrollment(unittest.TestCase):
    """
    Tests for Enterprise Enrollment model
    """

    def setUp(self):
        enterprise_user_id = 1234
        course_id = 'course-v1:edX+DemoX+DemoCourse'

        self.enrollment = EnterpriseEnrollmentFactory(enterprise_user_id=enterprise_user_id, course_id=course_id)
        super(TestEnterpriseEnrollment, self).setUp()

    @ddt.data(str, repr)
    def test_string_conversion(self, method):
        """
        Test conversion to string.
        """
        expected_str = '<Enterprise Enrollment for user 1234 in course-v1:edX+DemoX+DemoCourse>'
        assert expected_str == method(self.enrollment)
